#include <Wire.h>
#include <LiquidCrystal_I2C.h>


byte pin = 2;
float rev;
int rpm = 0;
int time;
int kmh;
float tiresize = 1.866;             //in meters
byte measurePoints = 2;             //how many points is sensor sensing per one cycle
float avg;
float current;
float dist;

byte statusLed    = 13;
byte sensorpin       = 3;
float calibrationFactor = 41.6666666667;      //calibraton factor from flowmeter datasheet
byte pulseCount;

float flowRate;
float flowLiters;
float totalLiters;
unsigned long oldTime;


LiquidCrystal_I2C lcd(0x27, 20, 4);

void isr()
{
  rev++;                                    //counting tire revolutions 
  dist += tiresize / measurePoints;         //counting total distance travelled
}

void pulseCounter()
{                           // Increment the pulse counter
  pulseCount++;
}


void setup() {
  pinMode(pin, INPUT);
  attachInterrupt(digitalpinToInterrupt(pin), isr, FALLING);
  lcd.init();
  lcd.backlight();


  pinMode(statusLed, OUTPUT);
  digitalWrite(statusLed, HIGH);
  pinMode(sensorpin, INPUT);
  digitalWrite(sensorpin, HIGH);
  pulseCount        = 0;
  flowRate          = 0.0;
  flowLiters        = 0;
  totalLiters       = 0;
  oldTime           = 0;

  attachInterrupt(digitalpinToInterrupt(sensorpin), pulseCounter, FALLING);
}

void loop() {

  if ((millis() - oldTime) > 1000)   // only process counters once per second
  {
    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print("RPM:");
    lcd.setCursor(0, 1);
    lcd.print("km/h:");
    lcd.setCursor(0, 2);
    lcd.print("l/min:");
    lcd.setCursor(0, 3);
    lcd.print("l:");

    detachInterrupt(digitalpinToInterrupt(pin));           //detaches the interrupt
    time = millis() - oldTime;                             //finds the time
    rpm = ((rev / time) * 60000) / measurePoints;          //calculates rpm
    rev = 0;
    kmh = (rpm * tiresize * 60) / 1000;                    //calculates current speed
    lcd.setCursor(6, 0);                                   //printing data on display
    lcd.print(rpm);
    lcd.setCursor(6, 1);
    lcd.print(kmh);




    attachInterrupt(digitalpinToInterrupt(pin), isr, FALLING);


    detachInterrupt(digitalpinToInterrupt(sensorpin));          
    flowRate = (1000.0 / (millis() - oldTime) * pulseCount) / calibrationFactor;              //calculate current flow rate
    oldTime = millis();
    flowLiters = (flowRate / 60);                                                             //counting current flow in liters per minute 
    totalLiters += flowLiters;                                                                //counting total flow volume in liters
    
    lcd.setCursor(6, 2);
    lcd.print(flowRate);
    lcd.setCursor(6, 3);
    lcd.print(totalLiters);
    pulseCount = 0;

    avg = totalLiters / dist * 100;                       //calculate average fuel consumption
    lcd.setCursor(15, 0);
    lcd.print(avg);

    current = ((flowLiters * 60) / kmh) * 100;            //calculate current fuel consumption
    lcd.setCursor(15, 1);
    lcd.print(current);


    attachInterrupt(digitalpinToInterrupt(sensorpin), pulseCounter, FALLING);
  }



}
